@@ -4,7 +4,7 @@ Copyright
     - Copyright: 2011-2014 Ethan Furman
     - Author: Ethan Furman
     - Contact: ethan@stoneleaf.us
-    - Version: 0.65.000 as of 2014-04-18
+    - Version: 0.73.004 as of 2014-07-07
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -44,6 +44,8 @@ system_sep = _os.path.sep
 
 pyver = float('%s.%s' % _sys.version_info[:2])
 
+version = 0, 73, 5
+
 class Path(object):
     """\
     vol = [ c: | //node/sharepoint | '' ]
@@ -97,18 +99,43 @@ def ext(self):
 methods['ext'] = property(ext)
 del ext
 
-def dir_pieces(self):
+def elements(self):
+    return list(self.iter())
+methods['elements'] = property(elements)
+del elements
+
+def dir_elements(self):
+    return list(self.iter_dirs())
+methods['dir_elements'] = property(dir_elements)
+del dir_elements
+
+def iter_all(self):
+    result = list(self.iter_dirs())
+    if self.vol:
+        result.insert(0, self.vol)
+    if self.filename:
+        result.append(self.filename)
+    return iter(result)
+methods['iter'] = iter_all
+del iter_all
+
+def iter_dirs(self):
     result = []
     cls = self.__class__
     if self._dirs:
         if self._dirs[0] == SEP:
             result = [cls(SEP)]
         dirs = self._dirs.strip(SEP)
-        result.extend([cls(d) for d in dirs.split(SEP)])
-    return result
-methods['dir_pieces'] = property(dir_pieces)
-del dir_pieces
+        if dirs:
+            result.extend([cls(d) for d in dirs.split(SEP)])
+    return iter(result)
+methods['iter_dirs'] = iter_dirs
+del iter_dirs
 
+def base_cls(self):
+    return self.__class__.__base__
+methods['base_cls'] = property(base_cls)
+del base_cls
 
 def __new__(cls, string='', sep=None):
     new_cls = cls.__bases__[1]  # either str or unicode
@@ -298,50 +325,25 @@ def __sub__(self, other):
     if not isinstance(other, String):
         return NotImplemented
     other = self.__class__(other)
-    o_vol, o_dirs, o_base, o_ext = other._vol, other._dirs, other._base, other._ext
-    s_vol, s_dirs, s_base, s_ext = self._vol, self._dirs, self._base, self._ext
-    if o_vol:
-        if o_vol != s_vol:
-            raise ValueError("cannot subtract %r from %r" % (o_vol, s_vol))
+    if other._vol:
+        if other._vol != self._vol:
+            raise ValueError("cannot subtract %r from %r" % (other, self))
         vol = ''
     else:
-        vol = s_vol
-    if o_dirs:
-        if not s_dirs.startswith(o_dirs):
-            raise ValueError("cannot subtract %r from %r" % (o_dirs, s_dirs))
-        dirs = s_dirs[len(o_dirs):]
-    else:
-        dirs = s_dirs
-    if o_base or o_ext:
-        o_filename = o_base + o_ext
-        if dirs.startswith(o_filename):
-            dirs = dirs[len(o_filename):]
-            o_base = o_ext = ''
-    if o_base:
-        if s_base.startswith(o_base):
-            base = s_base[len(o_base):]
-        else:
-            raise ValueError("cannot subtract %r from %r" % (o_base, dirs + s_base))
-    else:
-        base = s_base
-    if o_ext:
-        if s_ext.startswith(o_ext):
-            ext = s_ext[len(o_ext):]
-        else:
-            raise ValueError("cannot subtract %r from %r" % (o_ext, dirs + s_ext))
-    else:
-        ext = s_ext
-    if vol[:2] == SEP*2 and not dirs[:1] == SEP:
-        dirs = SEP + dirs
-    return self.__class__(vol + dirs + base + ext)
+        vol = self._vol
+    o = other.dirs + other.filename
+    s = self.dirs + self.filename
+    if not s.startswith(o):
+        raise ValueError("cannot subtract %r from %r" % (other, self))
+    return self.__class__(vol+s[len(o):])
 methods['__sub__'] = __sub__
 del __sub__
 
 def copy(self, dst):
     'thin wrapper around shutil.copy2'
     if isinstance(dst, self.__class__):
-        dst = str(dst)
-    src = str(self)
+        dst = self.base_cls(dst)
+    src = self.base_cls(self)
     _shutil.copy2(src, dst)
 methods['copy'] = copy
 del copy
@@ -349,8 +351,8 @@ del copy
 def copytree(self, dst, symlinks=False, ignore=None):
     'thin wrapper around shutil.copytree'
     if isinstance(dst, self.__class__):
-        dst = str(dst)
-    src = str(src)
+        dst = self.base_cls(dst)
+    src = self.base_cls(src)
     _shutil.copytree(src, dst, symlinks, ignore)
 methods['copytree'] = copytree
 del copytree
@@ -363,6 +365,18 @@ def count(self, sub, start=None, end=None):
 methods['count'] = count
 del count
 
+def chmod(self, mode):
+    "thin wrapper around os.chmod"
+    _os.chmod(self, mode)
+methods['chmod'] = chmod
+del chmod
+
+def chown(self, uid, gid):
+    "thin wrapper around os.chown"
+    _os.chown(self, uid, gid)
+methods['chown'] = chown
+del chown
+
 def endswith(self, suffix, start=None, end=None):
     if isinstance(suffix, String):
         new_suffix = suffix.replace(system_sep, SEP)
@@ -400,8 +414,8 @@ def format_map(self, other):
 methods['format_map'] = format_map
 del format_map
 
-def glob(self):
-    return [Path(p) for p in native_glob(self)]
+def glob(self, pattern=''):
+    return [Path(p) for p in native_glob(self + pattern)]
 methods['glob'] = glob
 del glob
 
@@ -444,33 +458,51 @@ def lstrip(self, chars=None):
 methods['lstrip'] = lstrip
 del lstrip
 
-def mkdir(self, mode=None):
+def mkdir(self, mode=None, owner=None):
+    """
+    Create a directory, setting owner if given.
+    """
     if mode is None:
         _os.mkdir(self)
     else:
         _os.mkdir(self, mode)
+    if owner is not None:
+        _os.chown(*owner)
 methods['mkdir'] = mkdir
 del mkdir
 
-def mkdirs(self, mode=None):
+def mkdirs(self, mode=None, owner=None):
+    """
+    Create any missing intermediate directories, setting owner if given.
+    """
     path = self.vol
-    segments = self.dir_pieces
-    for dir in segments:
+    elements = self.elements
+    for dir in elements:
         path /= dir
         if not path.exists():
-            self.mkdir(path, mode)
+            path.mkdir(mode, owner)
 methods['mkdirs'] = mkdirs
 del mkdirs
 
 def move(self, dst):
     'thin wrapper around shutil.move'
     if isinstance(dst, self.__class__):
-        dst = str(dst)
-    src = str(self)
+        dst = self.base_cls(dst)
+    src = self.base_cls(self)
+    print src, dst
     _shutil.move(src, dst)
 methods['move'] = move
 del move
 
+def rename(self, dst):
+    'thin wrapper around os.rename)'
+    if isinstance(dst, self.__class__):
+        dst = self.base_cls(dst)
+    src = self.base_cls(self)
+    _os.rename(src, dst)
+methods['rename'] = rename
+del rename
+
 def replace(self, old, new, count=None):
     old = old.replace(system_sep, SEP)
     new = new.replace(system_sep, SEP)
@@ -481,9 +513,15 @@ def replace(self, old, new, count=None):
 methods['replace'] = replace
 del replace
 
+def rmdir(self):
+    'thin wrapper around os.rmdir'
+    _os.rmdir(self)
+methods['rmdir'] = rmdir
+del rmdir
+
 def rmtree(self, ignore_errors=None, onerror=None):
     'thin wrapper around shutil.rmtree'
-    target = str(self)
+    target = self.base_cls(self)
     if ignore_errors is None and onerror is None:
         _shutil.rmtree(target)
     elif ignore_errors is not None and onerror is None:
